// objekat.h - Klasa apstraktnih objekata.

#ifndef _objekat_h_
#define _objekat_h_

#include <iostream>
#include <typeinfo>
using namespace std;

class Obj {
  static int ukId;       // Poslednje korisceni identifikator.
  const int id;          // Identifikator objekta.
  virtual bool jednako (const Obj &obj) const =0;       // Da li su jednaki?
  virtual void pisi  (ostream &d) const =0;             // Pisanje.
  virtual void citaj (istream &d) =0;                   // Citanje.
public:
  Obj (): id(++ukId) {}              // Novi objekat dobija novi ident.
  Obj (const Obj&): id(++ukId) {}    // Kopija dobija novi identifikator.
  Obj& operator= (const Obj&)        // Levom operandu se ne menja ident.
    { return *this; };
  virtual ~Obj () {}                                    // Unistavanje.
  const char *klasa () const                            // Ime klase.
    { return typeid(*this).name (); }
  virtual Obj *kopija () const =0;                      // Kopija objekta.
  friend bool operator== (const Obj &o1, const Obj &o2)      // o1 == o2
    { return typeid(o1)==typeid(o2) && o1.jednako(o2); }
  friend bool operator!= (const Obj &o1, const Obj &o2)      // o1 != o2
    { return ! (o1 == o2); }
  friend ostream& operator<< (ostream &d, const Obj &obj)    // d  << obj
    { d << obj.klasa()[0] << obj.id; obj.pisi (d); return d; }
  friend istream& operator>> (istream &d, Obj &obj)          // d  >> obj
    { obj.citaj (d); return d; }
};

#endif
