// vektor2.h - Klasa vektora realnih brojeva.

#ifndef _vektor2_h_
#define _vektor2_h_

class Vekt {                            // Atributi:
  int min, max;                         //   granice indeksa,
  int duz;                              //   broj komponenata,
  double* niz;                          //   niz komponenata.
                                        // Pomocne metode:
  void pravi (int poc, int kra);        //   stvaranje vektora,
  void kopiraj (const Vekt& v);         //   kopiranje vektora,
  void brisi () {delete[] niz; niz=0;}  //   unistavanje vektora.
public:                                 // Kodovi gresaka:
  enum Greska { OPSEG,                  //   neispravan opseg indeksa,
                PRAZAN,                 //   vektor je prazan,
                INDEKS,                 //   indeks je izvan opsega,
                DUZINA };               //   neusaglasene duzine vektora.
                                        // Konstruktori:
  Vekt () { niz = 0; }                  //         prazan niz,
  explicit Vekt (int kra) { pravi (1, kra); } //   podraz. pocetni indeks,
  Vekt (int poc, int kra) { pravi (poc,kra);} //   sa zadatim indeksima,
  Vekt (const Vekt& v) { kopiraj (v); } //         kopija vektora.
  ~Vekt () { brisi (); }                // Destruktor.
  Vekt& operator= (const Vekt& v) {     // Dodela vrednosti.
    if (this != &v) { brisi (); kopiraj (v); }
    return *this;
  }                                     // Pristup komponenti:
  double& operator[] (int ind) {        //      promenljivog vektora,
    if (niz == 0)           throw PRAZAN;
    if (ind<min || ind>max) throw INDEKS;
    return niz[ind-min];
  }
  const double& operator[] (int ind) const //   nepromenljivog vektora.
    { return const_cast<Vekt&>(*this)[ind]; }
  friend double operator*               // Skalarni proizvod.
    (const Vekt &v, const Vekt& w);
  int minInd () const { return min; }   // Najmanji indeks.
  int maxInd () const { return max; }   // Najveci indeks.
};

#endif
